@ECHO OFF
REM * =========================================================== *
REM * This script is placed into the public domain by its author. *
REM * All copyrights are hereby relinquished on the routines and  *
REM * data in this file.   -- James Pujals (DZ-Jay), 2015         *
REM * =========================================================== *

ECHO IntyBASIC Build Script (INTYBUILD.BAT)
ECHO   Created by:   DZ-Jay
ECHO   Last updated: 07/13/2015
ECHO.

SETLOCAL

REM ========================================
REM Initialize Build Variables
REM ========================================
SET BASEPATH=%INTYBASIC_INSTALL%
	REM IF NOT EXIST "%BASEPATH%" GOTO BAD-INSTALL

SET LIBDIR=%BASEPATH%\lib
SET BINDIR=%BASEPATH%\bin
SET COMP=%BINDIR%\IntyBASIC.exe
SET ASMB=%BINDIR%\as1600.exe

REM ========================================
REM Initialize Input Variables
REM ========================================
SET OLDDIR=%CD%
SET FDIR=%~dp1
SET FFULL=%~f1
SET FNAME=%~n1
SET FEXT=%~x1
SET FFNM=%~nx1
SET SRCDIR=%FDIR%asm
SET TRGDIR=%FDIR%bin

SET FASM=%FNAME%.asm
SET FLST=%FNAME%.ls
SET FSYM=%FNAME%.sym
SET FMAP=%FNAME%.map

SET TITLE=%~2

IF "%TITLE%"=="" SET TITLE=%FNAME%
IF NOT "%FEXT%"=="" GOTO TEST-INPUT
	SET FEXT=.BAS
	SET FFULL=%FFULL%%FEXT%

REM ========================================
REM Check Source File Arguments
REM ========================================
:TEST-INPUT
IF "%FNAME%"=="" GOTO BAD-FILENAME
IF NOT EXIST "%FFULL%" GOTO BAD-FILENAME
IF /I NOT "%FEXT%"==".BAS" GOTO BAD-EXTENSION


REM ========================================
REM Check Output Directories
REM ========================================
:TEST-SRCDIR
IF EXIST "%SRCDIR%" GOTO TEST-TRGDIR
	MD "%SRCDIR%"
	IF %ERRORLEVEL% NEQ 0 GOTO ERROR

:TEST-TRGDIR
IF EXIST "%TRGDIR%" GOTO BAS-COMPILE
	MD "%TRGDIR%"
	IF %ERRORLEVEL% NEQ 0 GOTO ERROR


REM ========================================
REM Build Output Binary
REM ========================================
:BAS-COMPILE
	CD "%FDIR%"
	"%COMP%" --title "%TITLE%" "%FFULL%" "%SRCDIR%\%FASM%" "%LIBDIR%"
	IF %ERRORLEVEL% NEQ 0 GOTO ERROR-BAS

:ASM-ASSEMBLE
	ECHO Assembling...
	"%ASMB%" -o "%TRGDIR%\%FNAME%.rom" -l "%TRGDIR%\%FLST%" -s "%TRGDIR%\%FSYM%" -j "%TRGDIR%\%FMAP%" "%SRCDIR%\%FASM%"

	IF %ERRORLEVEL% NEQ 0 GOTO ERROR-ASM

	ECHO.
	ECHO Program build completed.
	GOTO END

REM ========================================
REM Error Handlers
REM ========================================
:BAD-DOCSDIR
	ECHO Unable to determine "Documents" directory.
	GOTO ERROR

:BAD-INSTALL
	ECHO Unable to find IntyBASIC installation directory.
	GOTO ERROR

:BAD-EXTENSION
	ECHO Invalid file extension "%FEXT%"
	GOTO USAGE

:BAD-FILENAME
	ECHO File not found "%FFNM%"
	GOTO USAGE

:USAGE
	ECHO Usage: intybuild [FILE.BAS] ["TITLE"]
	GOTO END

:ERROR-BAS
	ECHO.
	ECHO IntyBASIC compilation failed.
	GOTO ERROR

:ERROR-ASM
	ECHO.
	ECHO AS1600 assemblage failed.
	GOTO ERROR

:ERROR
	ECHO Program build aborted.
	ENDLOCAL 
	SET ERRORLEVEL=1

:END
	ENDLOCAL 
	CD %OLDDIR%
	SET ERRORLEVEL=0
	ECHO.
